// software for version 2 of the PCB


#include "pwmLight.h"

const int LED_PINS[] = {5, 6, 9, 10, 4, 7};

const int STATUS_LED = 8;

const int NAV_LIGHTS = 0;
const int BEACON_LIGHT = 1;
const int LANDING_LIGHT_1 = 2;
const int LANDING_LIGHT_2 = 3;
const int STROBE_LIGHT_1 = 4;
const int STROBE_LIGHT_2 = 5;

const int SERVO_IN_1_PIN = 2;

int beaconPeriod = 2000;
int beaconOffset = 0; 
int strobePeriod = beaconPeriod;
int strobeOffset1 = strobePeriod/2; 
int strobeOffset2 = strobeOffset1/2; 
int strobeDuration = 60; 
int wigWagPeriod = 1400;

volatile long input1_StartTime = 0; 
volatile int input1_raw = 0;            // signal values 1000 - 2000 us
volatile long lastInput1_MsTime = 0; 
volatile long input2_StartTime = 0; 
volatile int input2_raw = 0;            // signal values 1000 - 2000 us
volatile long lastInput2_MsTime = 0; 

bool strobeOn = true; 
bool navLightsOn = true; 
bool beaconOn = true;
bool landingLightsOn = true; 
bool landingLightsWigWag = true;

PwmLight navLights(LED_PINS[NAV_LIGHTS], 20);
PwmLight beaconLight(LED_PINS[BEACON_LIGHT], 20);
PwmLight landingLight1(LED_PINS[LANDING_LIGHT_1], 5);
PwmLight landingLight2(LED_PINS[LANDING_LIGHT_2], 5);

void setup() {
  pinMode(SERVO_IN_1_PIN, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(SERVO_IN_1_PIN), input1_icr, CHANGE);
  
  for (int i = 0; i < 6; i++) 
    pinMode(LED_PINS[i], OUTPUT);

  pinMode(STATUS_LED, OUTPUT);
}

void input1_icr() {
  if (digitalRead(SERVO_IN_1_PIN)) {
    // signal is high
    input1_StartTime = micros(); 
  } else {
    input1_raw = micros() - input1_StartTime; 
    lastInput1_MsTime = millis(); 
  }
}

void loop() {

  if (lastInput1_MsTime + 50 < millis()) {
    // No signal is connected on input 1
    navLightsOn = true; 
    landingLightsOn = true; 
    landingLightsWigWag = true;
    strobeOn = true; 
    beaconOn = true;  
  } else {
    navLightsOn = (input1_raw > 1100); 
    landingLightsOn = (input1_raw > 1700); 
    landingLightsWigWag = (input1_raw > 1900);
    strobeOn = (input1_raw > 1500); 
    beaconOn = (input1_raw > 1300);   
  }

  
  if (navLightsOn) { 
    navLights.setIntencity(255); 
  } else {
    navLights.setIntencity(0); 
  }
  

  if (strobeOn) {       // Bright flashing white lights on the wing tips. Making double flashef for high visibility
    bool flash1 = (millis() + strobeOffset1) % strobePeriod < strobeDuration;
    bool flash2 = (millis() + strobeOffset1 + 120) % strobePeriod < strobeDuration;
    bool flash3 = (millis() + strobeOffset2) % strobePeriod < strobeDuration;
    bool flash4 = (millis() + strobeOffset2 + 120) % strobePeriod < strobeDuration;
    digitalWrite(LED_PINS[STROBE_LIGHT_1], flash1 || flash2);
    digitalWrite(LED_PINS[STROBE_LIGHT_2], flash3 || flash4);
  } else {
    digitalWrite(LED_PINS[STROBE_LIGHT_1], false);
    digitalWrite(LED_PINS[STROBE_LIGHT_2], false);
  }

  byte leftLandingLightValue = 0, rightLandingLightValue = 0;   
  if (landingLightsOn) {      // one bright white landing light on each side on the wing
    if (landingLightsWigWag) {  // alternate left and right landing light, used for high visibility
      if (millis() % wigWagPeriod < wigWagPeriod/2) {        
        leftLandingLightValue = 255; 
        rightLandingLightValue = 0; 
      } else {
        leftLandingLightValue = 0; 
        rightLandingLightValue = 255; 
      }      
    } else {
      leftLandingLightValue = rightLandingLightValue = 150; 
    }
  } 
  landingLight1.setIntencity(leftLandingLightValue);
  landingLight2.setIntencity(rightLandingLightValue); 

  // Softly flashing warning light, usually red. Placed on top of the vertical fin or fuselage on an airplane. 
  if (beaconOn && (millis() + beaconOffset) % beaconPeriod < wigWagPeriod/4) {
    beaconLight.setIntencity(255);   
  } else{
    beaconLight.setIntencity(0);
  }

  digitalWrite(STATUS_LED, millis() % 995 < 10);      // heartbeat status LED flashing

  delay(5);
}
